@@ -56,7 +56,7 @@
 const uint16_t MAX_VOICES = 32;			// polyphony
 const uint8_t  MAX_NOTES  = 128;
 
-const float MIN_ENV_MSECS = 2.0f;		// min 2msec per stage
+const float MIN_ENV_MSECS = 2.0f;		// min 2 msec per stage
 const float MAX_ENV_MSECS = 5000.0f;	// max 5 sec per stage (default)
 
 const float DETUNE_SCALE  = 0.5f;
@@ -339,6 +339,28 @@ struct synthv1_env
 		p->c0 = p->value;
 	}
 
+	void restart(State *p, bool legato)
+	{
+		p->running = true;
+		if (legato) {
+			p->stage = Decay;
+			p->frames = min_frames;
+			p->phase = 0.0f;
+			p->delta = 1.0f / float(p->frames);
+			p->c1 = *sustain - p->value;
+			p->c0 = 0.0f;
+		} else {
+			p->stage = Attack;
+			p->frames = uint32_t(*attack * *attack * max_frames);
+			p->phase = 0.0f;
+			if (p->frames < min_frames)
+				p->frames = min_frames;
+			p->delta = 1.0f / float(p->frames);
+			p->c1 = 1.0f;
+			p->c0 = 0.0f;
+		}
+	}
+
 	// parameters
 
 	synthv1_port attack;
@@ -1146,7 +1168,7 @@ void synthv1_impl::updateEnvTimes_1 (void)
 
 	float envtime_msecs = 10000.0f * m_dco1.envtime0;
 	if (envtime_msecs < MIN_ENV_MSECS)
-		envtime_msecs = MIN_ENV_MSECS + 1.0f;
+		envtime_msecs = MIN_ENV_MSECS * 2.0f;
 
 	const uint32_t min_frames = uint32_t(srate_ms * MIN_ENV_MSECS);
 	const uint32_t max_frames = uint32_t(srate_ms * envtime_msecs);
@@ -1169,7 +1191,7 @@ void synthv1_impl::updateEnvTimes_2 (void)
 
 	float envtime_msecs = 10000.0f * m_dco2.envtime0;
 	if (envtime_msecs < MIN_ENV_MSECS)
-		envtime_msecs = MIN_ENV_MSECS + 1.0f;
+		envtime_msecs = MIN_ENV_MSECS * 2.0f;
 
 	const uint32_t min_frames = uint32_t(srate_ms * MIN_ENV_MSECS);
 	const uint32_t max_frames = uint32_t(srate_ms * envtime_msecs);
@@ -1470,14 +1492,17 @@ void synthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 			if (on1) {
 				// mono voice modes
 				if (*m_def1.mono > 0.0f) {
+					int n1 = 0;
 					for (pv = m_play_list.next(); pv; pv = pv->next()) {
 						if (pv->note1 >= 0
 							&& pv->dca1_env.stage != synthv1_env::Release) {
 							m_dcf1.env.note_off_fast(&pv->dcf1_env);
 							m_lfo1.env.note_off_fast(&pv->lfo1_env);
 							m_dca1.env.note_off_fast(&pv->dca1_env);
-							m_note1[pv->note1] = NULL;
-							pv->note1 = -1;
+							if (++n1 > 1) { // there shall be only one
+								m_note1[pv->note1] = NULL;
+								pv->note1 = -1;
+							}
 						}
 					}
 				}
@@ -1496,14 +1521,17 @@ void synthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 			if (on2) {
 				// mono voice modes
 				if (*m_def2.mono > 0.0f) {
+					int n2 = 0;
 					for (pv = m_play_list.next(); pv; pv = pv->next()) {
 						if (pv->note2 >= 0
 							&& pv->dca2_env.stage != synthv1_env::Release) {
 							m_dcf2.env.note_off_fast(&pv->dcf2_env);
 							m_lfo2.env.note_off_fast(&pv->lfo2_env);
 							m_dca2.env.note_off_fast(&pv->dca2_env);
-							m_note2[pv->note2] = NULL;
-							pv->note2 = -1;
+							if (++n2 > 1) { // there shall be only one
+								m_note2[pv->note2] = NULL;
+								pv->note2 = -1;
+							}
 						}
 					}
 				}
@@ -1652,6 +1680,19 @@ void synthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 							m_dcf1.env.note_off(&pv->dcf1_env);
 							m_lfo1.env.note_off(&pv->lfo1_env);
 						}
+						m_note1[pv->note1] = NULL;
+						pv->note1 = -1;
+						// mono legato?
+						if (*m_def1.mono > 0.0f) {
+							do pv = pv->prev();	while (pv && pv->note1 < 0);
+							if (pv && pv->note1 >= 0) {
+								const bool legato1 = (*m_def1.mono > 1.0f);
+								m_dcf1.env.restart(&pv->dcf1_env, legato1);
+								m_lfo1.env.restart(&pv->lfo1_env, legato1);
+								m_dca1.env.restart(&pv->dca1_env, legato1);
+								m_note1[pv->note1] = pv;
+							}
+						}
 					}
 				}
 			}
@@ -1667,6 +1708,19 @@ void synthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 							m_dcf2.env.note_off(&pv->dcf2_env);
 							m_lfo2.env.note_off(&pv->lfo2_env);
 						}
+						m_note2[pv->note2] = NULL;
+						pv->note2 = -1;
+						// mono legato?
+						if (*m_def2.mono > 0.0f) {
+							do pv = pv->prev();	while (pv && pv->note2 < 0);
+							if (pv && pv->note2 >= 0) {
+								const bool legato2 = (*m_def2.mono > 1.0f);
+								m_dcf2.env.restart(&pv->dcf2_env, legato2);
+								m_lfo2.env.restart(&pv->lfo2_env, legato2);
+								m_dca2.env.restart(&pv->dca2_env, legato2);
+								m_note2[pv->note2] = pv;
+							}
+						}
 					}
 				}
 			}
@@ -1866,6 +1920,8 @@ void synthv1_impl::allSustainOff_1 (void)
 				m_dca1.env.note_off(&pv->dca1_env);
 				m_dcf1.env.note_off(&pv->dcf1_env);
 				m_lfo1.env.note_off(&pv->lfo1_env);
+				m_note1[pv->note1] = NULL;
+				pv->note1 = -1;
 			}
 		}
 		pv = pv->next();
@@ -1883,6 +1939,8 @@ void synthv1_impl::allSustainOff_2 (void)
 				m_dca2.env.note_off(&pv->dca2_env);
 				m_dcf2.env.note_off(&pv->dcf2_env);
 				m_lfo2.env.note_off(&pv->lfo2_env);
+				m_note2[pv->note2] = NULL;
+				pv->note2 = -1;
 			}
 		}
 		pv = pv->next();
@@ -2292,11 +2350,8 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 			if (pv->dca1_env.stage == synthv1_env::Idle &&
 				pv->dca2_env.stage == synthv1_env::Idle) {
-				if (pv->note1 >= 0)
-					m_note1[pv->note1] = NULL;
-				if (pv->note2 >= 0)
-					m_note2[pv->note2] = NULL;
-				free_voice(pv);
+				if (pv->note1 < 0 && pv->note2 < 0)
+					free_voice(pv);
 				nblock = 0;
 			} else {
 				if (pv->dcf1_env.running && pv->dcf1_env.frames == 0)
