@@ -1,7 +1,7 @@
 // synthv1_param.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -103,7 +103,7 @@ struct ParamInfo {
 	{ "DEF1_PRESSURE", PARAM_FLOAT,   0.2f,   0.0f,   1.0f }, // DEF1 Pressure
 	{ "DEF1_VELOCITY", PARAM_FLOAT,   0.2f,   0.0f,   1.0f }, // DEF1 Velocity
 	{ "DEF1_CHANNEL",  PARAM_INT,     0.0f,   0.0f,  16.0f }, // DEF1 Channel
-	{ "DEF1_MONO",     PARAM_BOOL,    0.0f,   0.0f,   1.0f }, // DEF1 Mono
+	{ "DEF1_MONO",     PARAM_INT,     0.0f,   0.0f,   2.0f }, // DEF1 Mono
 
 	{ "DCO2_SHAPE1",   PARAM_INT,     1.0f,   0.0f,   4.0f }, // DCO2 Wave Shape 1
 	{ "DCO2_WIDTH1",   PARAM_FLOAT,   1.0f,   0.0f,   1.0f }, // DCO2 Wave Width 1
@@ -160,7 +160,7 @@ struct ParamInfo {
 	{ "DEF2_PRESSURE", PARAM_FLOAT,   0.2f,   0.0f,   1.0f }, // DEF2 Pressure
 	{ "DEF2_VELOCITY", PARAM_FLOAT,   0.2f,   0.0f,   1.0f }, // DEF2 Velocity
 	{ "DEF2_CHANNEL",  PARAM_INT,     0.0f,   0.0f,  16.0f }, // DEF2 Channel
-	{ "DEF2_MONO",     PARAM_BOOL,    0.0f,   0.0f,   1.0f }, // DEF2 Mono
+	{ "DEF2_MONO",     PARAM_INT,     0.0f,   0.0f,   2.0f }, // DEF2 Mono
 
 	{ "CHO1_WET",      PARAM_FLOAT,   0.0f,   0.0f,   1.0f }, // Chorus Wet
 	{ "CHO1_DELAY",    PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // Chorus Delay
@@ -242,10 +242,11 @@ bool synthv1_param::paramFloat ( synthv1::ParamIndex index )
 
 
 // Preset serialization methods.
-void synthv1_param::loadPreset ( synthv1 *pSynth, const QString& sFilename )
+bool synthv1_param::loadPreset (
+	synthv1 *pSynth, const QString& sFilename )
 {
 	if (pSynth == NULL)
-		return;
+		return false;
 
 	QFileInfo fi(sFilename);
 	if (!fi.exists()) {
@@ -254,16 +255,16 @@ void synthv1_param::loadPreset ( synthv1 *pSynth, const QString& sFilename )
 			const QString& sPresetFile
 				= pConfig->presetFile(sFilename);
 			if (sPresetFile.isEmpty())
-				return;
+				return false;
 			fi.setFile(sPresetFile);
 			if (!fi.exists())
-				return;
+				return false;
 		}
 	}
 
 	QFile file(fi.filePath());
 	if (!file.open(QIODevice::ReadOnly))
-		return;
+		return false;
 
 	static QHash<QString, synthv1::ParamIndex> s_hash;
 	if (s_hash.isEmpty()) {
@@ -273,6 +274,9 @@ void synthv1_param::loadPreset ( synthv1 *pSynth, const QString& sFilename )
 		}
 	}
 
+	const QDir currentDir(QDir::current());
+	QDir::setCurrent(fi.absolutePath());
+
 	QDomDocument doc(SYNTHV1_TITLE);
 	if (doc.setContent(&file)) {
 		QDomElement ePreset = doc.documentElement();
@@ -312,15 +316,22 @@ void synthv1_param::loadPreset ( synthv1 *pSynth, const QString& sFilename )
 	file.close();
 
 	pSynth->reset();
+
+	QDir::setCurrent(currentDir.absolutePath());
+
+	return true;
 }
 
 
-void synthv1_param::savePreset ( synthv1 *pSynth, const QString& sFilename )
+bool synthv1_param::savePreset (
+	synthv1 *pSynth, const QString& sFilename, bool /*bSymLink*/ )
 {
 	if (pSynth == NULL)
-		return;
+		return false;
 
 	const QFileInfo fi(sFilename);
+	const QDir currentDir(QDir::current());
+	QDir::setCurrent(fi.absolutePath());
 
 	QDomDocument doc(SYNTHV1_TITLE);
 	QDomElement ePreset = doc.createElement("preset");
@@ -341,10 +352,15 @@ void synthv1_param::savePreset ( synthv1 *pSynth, const QString& sFilename )
 	doc.appendChild(ePreset);
 
 	QFile file(fi.filePath());
-	if (file.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
-		QTextStream(&file) << doc.toString();
-		file.close();
-	}
+	if (!file.open(QIODevice::WriteOnly | QIODevice::Truncate))
+		return false;
+
+	QTextStream(&file) << doc.toString();
+	file.close();
+
+	QDir::setCurrent(currentDir.absolutePath());
+
+	return true;
 }
 
 
