@@ -31,6 +31,8 @@
 #include "synthv1_controls.h"
 #include "synthv1_programs.h"
 
+#include "ui_synthv1widget.h"
+
 #include <QMessageBox>
 #include <QDir>
 #include <QTimer>
@@ -45,7 +47,7 @@
 
 // Constructor.
 synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
-	: QWidget(pParent, wflags)
+	: QWidget(pParent, wflags), p_ui(new Ui::synthv1widget), m_ui(*p_ui)
 {
 	Q_INIT_RESOURCE(synthv1);
 
@@ -252,14 +254,16 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	m_ui.Def1ChannelKnob->insertItems(0, channels);
 	m_ui.Def2ChannelKnob->insertItems(0, channels);
-#if 0
-	// Mono switches
-	m_ui.Def1MonoKnob->insertItems(0, states);
-	m_ui.Def2MonoKnob->insertItems(0, states);
-#else
-	m_ui.Def1MonoKnob->setAlignment(Qt::AlignHCenter | Qt::AlignBottom);
-	m_ui.Def2MonoKnob->setAlignment(Qt::AlignHCenter | Qt::AlignBottom);
-#endif
+
+	// Mono switches.
+	QStringList modes;
+	modes << tr("Poly");
+	modes << tr("Mono");
+	modes << tr("Legato");
+
+	m_ui.Def1MonoKnob->insertItems(0, modes);
+	m_ui.Def2MonoKnob->insertItems(0, modes);
+
 	// Output (stereo-)width limits.
 	m_ui.Out1WidthKnob->setMinimum(-1.0f);
 	m_ui.Out1WidthKnob->setMaximum(+1.0f);
@@ -804,6 +808,8 @@ synthv1widget::~synthv1widget (void)
 {
 	if (m_sched_notifier)
 		delete m_sched_notifier;
+
+	delete p_ui;
 }
 
 
@@ -1142,7 +1148,7 @@ void synthv1widget::loadPreset ( const QString& sFilename )
 
 	synthv1_ui *pSynthUi = ui_instance();
 	if (pSynthUi)
-		synthv1_param::loadPreset(pSynthUi->instance(), sFilename);
+		pSynthUi->loadPreset(sFilename);
 
 	updateLoadPreset(QFileInfo(sFilename).completeBaseName());
 }
@@ -1156,7 +1162,7 @@ void synthv1widget::savePreset ( const QString& sFilename )
 
 	synthv1_ui *pSynthUi = ui_instance();
 	if (pSynthUi)
-		synthv1_param::savePreset(pSynthUi->instance(), sFilename);
+		pSynthUi->savePreset(sFilename);
 
 	const QString& sPreset
 		= QFileInfo(sFilename).completeBaseName();
